@@ -20,6 +20,7 @@ import tempfile
 from gzip import GzipFile
 from zipfile import ZipFile
 from tarfile import TarFile
+import shutil
 
 from six import BytesIO
 
@@ -80,6 +81,17 @@ class ProjectData(XNATBaseObject):
                            secondary_lookup_field='label',
                            xsi_type='xnat:resourceCatalog')
 
+    def create_resource(self, label, format=None, data_dir=None, method=None):
+        uri = '{}/resources/{}'.format(self.fulluri, label)
+        self.xnat_session.put(uri, format=format)
+        self.clearcache()  # The resources changed, so we have to clear the cache
+        resource = self.xnat_session.create_object(uri, type_='xnat:resourceCatalog')
+
+        if data_dir is not None:
+            resource.upload_dir(data_dir, method=method)
+
+        return resource
+
     def download_dir(self, target_dir, verbose=True, progress_callback=None):
         """
         Download the entire project and unpack it in a given directory. Note
@@ -110,6 +122,15 @@ class ProjectData(XNATBaseObject):
             self.logger.info('Downloaded project to {}'.format(project_dir))
 
 
+class InvestigatorData(XNATBaseObject):
+    def __str__(self):
+        title = self.title or ''
+        first = self.firstname or ''
+        last = self.lastname or ''
+        fullname = '{title} {first} {last}'.format(title=title, first=first, last=last).strip()
+        return '<InvestigatorData {fullname}>'.format(fullname=fullname)
+
+
 class SubjectData(XNATBaseObject):
     SECONDARY_LOOKUP_FIELD = 'label'
 
@@ -198,6 +219,17 @@ class SubjectData(XNATBaseObject):
         self.xnat_session.put(share_uri, query=query)
         self.clearcache()
 
+    def create_resource(self, label, format=None, data_dir=None, method=None):
+        uri = '{}/resources/{}'.format(self.fulluri, label)
+        self.xnat_session.put(uri, format=format)
+        self.clearcache()  # The resources changed, so we have to clear the cache
+        resource = self.xnat_session.create_object(uri, type_='xnat:resourceCatalog')
+
+        if data_dir is not None:
+            resource.upload_dir(data_dir, method=method)
+
+        return resource
+
 
 class ExperimentData(XNATBaseObject):
     SECONDARY_LOOKUP_FIELD = 'label'
@@ -473,7 +505,7 @@ class AbstractResource(XNATBaseObject):
         # FIXME: ugly hack because direct query fails
         uri, label = self.uri.rsplit('/', 1)
         data = self.xnat_session.get_json(uri)['ResultSet']['Result']
-        
+
         def _guess_key( d ):
             if 'URI' not in d and 'ID' not in d and 'xnat_abstractresource_id' in d:
                 # HACK: This is a Resource where the label is not part of the uri, it uses this xnat_abstractresource_id instead.
@@ -509,6 +541,12 @@ class AbstractResource(XNATBaseObject):
         else:
             return int(file_count)
 
+    def refresh_catalog(self):
+        """
+        Call refresh catalog on this object, see :py:meth:`xnat.services.Services.refresh_catalog` for details.
+        """
+        self.xnat_session.services.refresh_catalog(self)
+
     @property
     @caching
     def files(self):
@@ -522,7 +560,7 @@ class AbstractResource(XNATBaseObject):
     def download(self, path, verbose=True):
         self.xnat_session.download_zip(self.uri + '/files', path, verbose=verbose)
 
-    def download_dir(self, target_dir, verbose=True):
+    def download_dir(self, target_dir, verbose=True, flatten_dirs=False):
         """
         Download the entire resource and unpack it in a given directory
 
@@ -534,15 +572,63 @@ class AbstractResource(XNATBaseObject):
 
             with ZipFile(temp_path) as zip_file:
                 zip_file.extractall(target_dir)
+                extracted_files = zip_file.namelist()
+
+            extraction_sub_directories = [os.path.dirname(os.path.normpath(i_extracted_file)) for i_extracted_file in extracted_files]
+            unique_extraction_sub_directories = list(set(extraction_sub_directories))
+            # Check if we have multiple resources
+            multiple_resources = len(unique_extraction_sub_directories) > 1
+
+            if flatten_dirs:
+                for i_extracted_file in extracted_files:
+                    new_resource_path = target_dir
+                    if multiple_resources:
+                        # With multiple resources we keep the subfolder
+                        new_resource_path = os.path.join(new_resource_path, os.path.basename(os.path.dirname(os.path.normpath(i_extracted_file))))
+                    if not os.path.exists(new_resource_path):
+                        os.makedirs(new_resource_path)
+
+                    new_resource_path = os.path.join(new_resource_path,  os.path.basename(os.path.normpath(i_extracted_file)))
+
+                    shutil.move(os.path.join(target_dir, i_extracted_file), new_resource_path)
+
+                # Remove the original download directory
+                root_extraction_sub_dir = os.path.join(target_dir, os.path.normpath(extraction_sub_directories[0]).split(os.sep)[0])
+                shutil.rmtree(root_extraction_sub_dir)
+                scan_directory = target_dir
+            else:
+                if multiple_resources:
+                    scan_directory = os.path.join(target_dir, os.path.dirname(os.path.normpath(unique_extraction_sub_directories[0])))
+                else:
+                    scan_directory = os.path.join(target_dir, unique_extraction_sub_directories[0])
 
         if verbose:
-            self.logger.info('Downloaded resource data to {}'.format(target_dir))
+            self.logger.info('Downloaded resource data to {}'.format(scan_directory))
+        return scan_directory
 
-    def upload(self, data, remotepath, overwrite=False, extract=False, **kwargs):
+    def upload(self, data, remotepath, overwrite=False, extract=False, file_content=None, file_format=None, file_tags=None, **kwargs):
+        """
+        Upload a file as an XNAT resource.
+
+        :param str data: The path to the file to upload
+        :param str remotepath: The remote path to which to uploadt
+        :param bool overwrite: Flag to force overwriting of files
+        :param bool extract: Extract the files on the XNAT server
+        :param str file_content: Set the Content of the file on XNAT
+        :param str file_format: Set the format of the file on XNAT
+        :param str file_tags: Set the tags of the file on XNAT
+        """
         uri = '{}/files/{}'.format(self.uri, remotepath.lstrip('/'))
         query = {}
         if extract:
             query['extract'] = 'true'
+        if file_content is not None:
+            query['content'] = file_content
+        if file_format is not None:
+            query['format'] = file_format
+        if file_tags is not None:
+            query['tags'] = file_tags
+
         self.xnat_session.upload(uri, data, overwrite=overwrite, query=query, **kwargs)
         self.files.clearcache()
 
