@@ -24,8 +24,10 @@ from zipfile import ZipFile
 
 from six import BytesIO
 
+from .core import XNATBaseObject
 from .prearchive import PrearchiveSession
 from .exceptions import XNATResponseError, XNATValueError
+from .mixin import ProjectData, SubjectData, ExperimentData
 
 TokenResult = collections.namedtuple('TokenResult', ('alias', 'secret'))
 
@@ -94,6 +96,27 @@ class Services(object):
     def xnat_session(self):
         return self._xnat_session
 
+    def guess_content_type(self, path):
+        content_type, _ = mimetypes.guess_type(path)
+
+        if content_type not in ['application/x-tar', 'application/zip']:
+            if path.endswith('.zip'):
+                self.xnat_session.logger.warning(
+                    'Found unexpect content type, but assuming zip based on the extension'
+                )
+                content_type = 'application/zip'
+            elif path.endswith('.tar.gz'):
+                self.xnat_session.logger.warning(
+                    'Found unexpect content type, but assuming tar based on the extension'
+                )
+                content_type = 'application/x-tar'
+            else:
+                self.xnat_session.logger.warning(
+                    'Found unexpected content (found type {}), this could result in errors on import!'.format(content_type)
+                )
+
+        return content_type
+
     def dicom_dump(self, src, fields=None):
         """
         Retrieve a dicom dump as a JSON data structure
@@ -165,12 +188,18 @@ class Services(object):
             query['dest'] = destination
 
         if project is not None:
+            if isinstance(project, ProjectData):
+                project = project.id
             query['project'] = project
 
         if subject is not None:
+            if isinstance(subject, SubjectData):
+                subject = subject.label
             query['subject'] = subject
 
         if experiment is not None:
+            if isinstance(experiment, ExperimentData):
+                experiment = experiment.label
             query['session'] = experiment
 
         if import_handler is not None:
@@ -178,23 +207,7 @@ class Services(object):
 
         # Get mimetype of file
         if content_type is None:
-            content_type, _ = mimetypes.guess_type(path)
-
-        if content_type not in ['application/x-tar', 'application/zip']:
-            if path.endswith('.zip'):
-                self.xnat_session.logger.warning(
-                    'Found unexpect content type, but assuming zip based on the extension'
-                )
-                content_type = 'application/zip'
-            elif path.endswith('.tar.gz'):
-                self.xnat_session.logger.warning(
-                    'Found unexpect content type, but assuming tar based on the extension'
-                )
-                content_type = 'application/x-tar'
-            else:
-                self.xnat_session.logger.warning(
-                    'Found unexpected content (found type {}), this could result in errors on import!'.format(content_type)
-                )
+            content_type = self.guess_content_type(path)
 
         uri = '/data/services/import'
         response = self.xnat_session.upload(uri=uri, file_=path, query=query, content_type=content_type, method='post')
@@ -229,7 +242,7 @@ class Services(object):
         The method has 2 options, default is zip_file:
 
         #. ``zip_file``: Create a temporary zip file and upload that
-        #. ``zip_file``: Create a temporary zip file in memory and upload it
+        #. ``zip_memory``: Create a temporary zip file in memory and upload it
 
         The considerations are that sometimes you can fit things in memory so
         you can save disk IO by putting it in memory.
@@ -260,8 +273,8 @@ class Services(object):
             return
         fh.seek(0)
         session = self.import_(fh, overwrite, quarantine, destination,
-                trigger_pipelines, project, subject,
-                experiment, content_type, import_handler=import_handler)
+                               trigger_pipelines, project, subject,
+                               experiment, content_type, import_handler=import_handler)
         fh.close()
         return session
 
@@ -293,16 +306,22 @@ class Services(object):
             query['cleanupAfterImport'] = 'false'
 
         if project is not None:
+            if isinstance(project, ProjectData):
+                project = project.id
             query['PROJECT_ID'] = project
 
         if subject is not None:
+            if isinstance(subject, SubjectData):
+                subject = subject.id
             query['SUBJECT_ID'] = subject
 
         if experiment is not None:
+            if isinstance(experiment, ExperimentData):
+                experiment = experiment.label
             query['EXPT_LABEL'] = experiment
 
         uri = '/data/services/import'
-        response = self.xnat_session.post(uri=uri, query=query)
+        response = self.xnat_session.post(uri, query=query)
 
         if response.status_code != 200:
             raise XNATResponseError('The response for uploading was ({}) {}'.format(response.status_code, response.text))
@@ -330,3 +349,55 @@ class Services(object):
         result = self.xnat_session.get_json(uri)
 
         return TokenResult(result['alias'], result['secret'])
+
+    def refresh_catalog(self, resource, checksum=False, delete=False, append=False, populate_stats=False):
+        """
+        Call for a refresh of the catalog, see https://wiki.xnat.org/display/XAPI/Catalog+Refresh+API for
+        details.
+
+        Introduced with XNAT 1.6.2, the refresh catalog service is used to update catalog xmls that are out of
+        sync with the file system.  This service can be used to store checksums for entries that are missing the,
+        remove entries that no longer have valid files, or add new entries for files that have been manually
+        added to the archive directory.
+
+        When using this feature to add files that have been manually added to the archive directory, you must
+        have placed the files in the appropriate archive directory (in the same directory as the generated
+        catalog xml or a sub-directory).  The catalog xml should already exist before triggering this service.
+        If you haven't generated the catalog yet, you can do so by doing a PUT to the resource URL
+        (i.e. /data/archive/experiments/ID/resources/TEST).
+
+        Extra parameters indicate operations to perform on the specified resource(s) during the refresh.
+        If non are given, then the catalog will be reviewed and updated for validity, but nothing else.
+
+        :param resource: XNATObject or uri indicating the resource to use
+        :param bool checksum: generate checksums for any entries that are missing them
+        :param bool delete: remove entries that do not reference valid files
+        :param bool append: add entries for files in the catalog directory (or sub-directory)
+        :param bool populate_stats: updates the statistics for the resource in the XNAT abstract resource table.
+        :return:
+        """
+        if isinstance(resource, XNATBaseObject):
+            resource = resource.fulluri
+
+        options = []
+
+        # Check which options to add
+        if checksum:
+            options.append('checksum')
+
+        if delete:
+            options.append('delete')
+
+        if append:
+            options.append('append')
+
+        if populate_stats:
+            options.append('populateStats')
+
+        # Make query that only contains options if any are given
+        query = {'resource': resource}
+
+        if options:
+            query['options'] = options
+
+        self.xnat_session.post('/data/services/refresh/catalog', query=query)
